<?php

namespace Hrm\Manpower\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Hrm\Manpower\Http\Requests\ManpowerRequest;
use Hrm\Manpower\Interfaces\ManpowerRepositoryInterface;
use Hrm\Manpower\Models\Manpower;

/**
 * Resource controller class for manpower.
 */
class ManpowerResourceController extends BaseController
{

    /**
     * Initialize manpower resource controller.
     *
     * @param type ManpowerRepositoryInterface $manpower
     *
     * @return null
     */
    public function __construct(ManpowerRepositoryInterface $manpower)
    {
        parent::__construct();
        $this->repository = $manpower;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Hrm\Manpower\Repositories\Criteria\ManpowerResourceCriteria::class);
    }

    /**
     * Display a list of manpower.
     *
     * @return Response
     */
    public function index(ManpowerRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Hrm\Manpower\Repositories\Presenter\ManpowerPresenter::class)
                ->$function();
        }

        $manpowers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('manpower::manpower.names'))
            ->view('manpower::manpower.index', true)
            ->data(compact('manpowers', 'view'))
            ->output();
    }

    /**
     * Display manpower.
     *
     * @param Request $request
     * @param Model   $manpower
     *
     * @return Response
     */
    public function show(ManpowerRequest $request, Manpower $manpower)
    {

        if ($manpower->exists) {
            $view = 'manpower::manpower.show';
        } else {
            $view = 'manpower::manpower.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('manpower::manpower.name'))
            ->data(compact('manpower'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new manpower.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ManpowerRequest $request)
    {

        $manpower = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('manpower::manpower.name')) 
            ->view('manpower::manpower.create', true) 
            ->data(compact('manpower'))
            ->output();
    }

    /**
     * Create new manpower.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ManpowerRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $manpower                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('manpower::manpower.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('manpower/manpower/' . $manpower->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/manpower/manpower'))
                ->redirect();
        }

    }

    /**
     * Show manpower for editing.
     *
     * @param Request $request
     * @param Model   $manpower
     *
     * @return Response
     */
    public function edit(ManpowerRequest $request, Manpower $manpower)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('manpower::manpower.name'))
            ->view('manpower::manpower.edit', true)
            ->data(compact('manpower'))
            ->output();
    }

    /**
     * Update the manpower.
     *
     * @param Request $request
     * @param Model   $manpower
     *
     * @return Response
     */
    public function update(ManpowerRequest $request, Manpower $manpower)
    {
        try {
            $attributes = $request->all();

            $manpower->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('manpower::manpower.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('manpower/manpower/' . $manpower->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('manpower/manpower/' . $manpower->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the manpower.
     *
     * @param Model   $manpower
     *
     * @return Response
     */
    public function destroy(ManpowerRequest $request, Manpower $manpower)
    {
        try {

            $manpower->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('manpower::manpower.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('manpower/manpower/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('manpower/manpower/' . $manpower->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple manpower.
     *
     * @param Model   $manpower
     *
     * @return Response
     */
    public function delete(ManpowerRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('manpower::manpower.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('manpower/manpower'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/manpower/manpower'))
                ->redirect();
        }

    }

    /**
     * Restore deleted manpowers.
     *
     * @param Model   $manpower
     *
     * @return Response
     */
    public function restore(ManpowerRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('manpower::manpower.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/manpower/manpower'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/manpower/manpower/'))
                ->redirect();
        }

    }

}
